/*********************************************************************************/
/* $Id: Boot.c,v 2.00 2002/05/24 21:59:30 Anoosh Naderi Exp $                    */
/*										 */
/* Copyright (c) 2001, Analog Devices Inc., All Rights Reserved			 */
/*										 */
/* Boot.c									 */
/*										 */
/* Code responsible for booting the modem					 */
/*										 */
/* This file is part of the "ADI USB ADSL Driver for Linux".			 */
/*										 */
/* "ADI USB ADSL Driver for Linux" is free software; you can redistribute it     */
/* and/or modify it under the terms of the GNU General Public License as         */
/* published by the Free Software Foundation; either version 2 of the License,   */
/* or (at your option) any later version.					 */
/*										 */
/* "ADI USB ADSL Driver for Linux" is distributed in the hope that it will be    */
/* useful, but WITHOUT ANY WARRANTY; without even the implied warranty of	 */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the		 */
/* GNU General Public License for more details.					 */
/*										 */
/* You should have received a copy of the GNU General Public License		 */
/* along with "ADI USB ADSL Driver for Linux"; if not, write to the Free Software*/
/* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA     */ 
/*********************************************************************************/

#include "Adiutil.h"

/*********************************************************************************/
/* Local only prototypes							 */
/*********************************************************************************/
static void IdmaUploadMainPage(Hardware *pHw);
static int IDMAWrite(Hardware *pHw, void *pData, UInt32 Size);
static void IDMACompletion(struct urb *urb);

/*********************************************************************************/
/* BootModem									 */
/* Kinda like doing a ctrl-alt-delete on your PC. Reuploads (or does first	 */
/* upload) of the DSP code, and initializes the host's state machine. 		 */
/*********************************************************************************/

#define IFERR_STOPBOOT(_stat_, _msg_) \
    if (_stat_ != USB_ST_NOERROR) \
    { \
        ZAP(DEBUG_BOOT, DEBUG_ERRORS, _msg_); \
        goto BootEnd; \
    }

// By Mani ///////////////////////////////////////////////////////////////////////////////
// This functions 1- Clears mail boxes							//
//		  2- Sends CNTL0.0 to modem						//
//		  3- Loads main page of DSP software to modem				//
//		  4- Leave control for state machine to send necessary CMVs after 	//
//                   getting modem ready async message					//
// This functions doesn't send reset command to modem as in Hard Reset			//
void SoftRestart(Hardware *pHw)								//
{											//
    UInt16 MsgCNTL00[]     = { 0x535C, 0x1110, 0xDCDC, 0x434E , 0x544C ,0 , 0, 0 };	//	    
    MsgSend(pHw,MsgCNTL00);								//
    wait_ms(40);     									//
    pHw->BootState = kBootModem_Kickoff;						//
    wake_up_interruptible(&(pHw->wq2));							//
}											//
//////////////////////////////////////////////////////////////////////////////////////////


void BootMachineCompletionHandler(Hardware *ptr,long wait);
long BootMachine(Hardware *pHw);


// By Mani ///////////////////////////////////////////////////////////////////////////////
// Boot Timer just wakes up the tboother thread 					//
											//
void BootTimerHandler(unsigned long ptr)						//
{											//
  Hardware *pHw = (Hardware *)ptr; 							//
											//
  wake_up_interruptible(&(pHw->wq2));							//
}											//
											//
// The booter thread does the next step in boot process and sleeps. if the step is the 	//
// last step ther thread sleeps untill the next boot start.				//
int booter_thread(void *data)								//
{											//
 long wait=0;										//
 Hardware *pHw = (Hardware *)data; 							//
 //printk(KERN_EMERG "Entering booter thread\n"); 					//
 while (1)										//
 {											//
  interruptible_sleep_on(&(pHw->wq2)); 							//
  wait = BootMachine(pHw);								//
  BootMachineCompletionHandler(pHw,wait);						//
 }											//
}											//
											//
// BootMachine does the next step (according to BootState) of booting procedure	and 	//
// returns the time that kernel shoul wait for the step being completed			//
long BootMachine(Hardware *pHw)								//
{											//
  int stat = USB_ST_NOERROR;								//
  UInt16 myWord;									//
  long wait=MSEC_TO_JIFFIES(0);								//
											//
  switch (pHw->BootState)								//
  {											//
     case kBootModem_Kickoff:								//
       pHw->BlockSwapReq = FALSE;                                           		//
       pHw->UserCmvApiAcceptable = FALSE;                                   		//
       stat = AdiCustomWrite(pHw, kADI_CMD_SET_MODE, kMODE_BOOTMODE_IDMA, 0, NULL);	//
       if (stat!=USB_ST_NOERROR) wait=-1;						//
       break;										//
     case kBootModem_Step2:								//
       stat = AdiCustomWrite(pHw, kADI_CMD_SET_MODE, kMODE_LOOPBACK_ON, 0, NULL);	//
       if (stat!=USB_ST_NOERROR) wait=-1; 						//
       break;										//
     case kBootModem_Step3:								//
        pHw->BlockSwapReq = FALSE;                                           		//
        pHw->UserCmvApiAcceptable = FALSE;                                   		//
        stat = AdiCustomWrite(pHw, kADI_CMD_SET_MODE, kMODE_BOOTMODE_IDMA, 0, NULL);	//
        if (stat!=USB_ST_NOERROR) wait=-1;						//
        break;										//
     case kBootModem_Step4:								//
        // Wait 200ms									//
        wait = MSEC_TO_JIFFIES(200);							//
        break;										//
     case kBootModem_Step5:								//
        stat = AdiCustomWrite(pHw, kADI_CMD_SET_MODE, kMODE_START_RESET, 0, NULL);	//
        if (stat!=USB_ST_NOERROR) wait=-1;						//
        break;										//
     case kBootModem_Step6:								//
        // Wait 200ms									//
        wait = MSEC_TO_JIFFIES(200);							//
        break;										//
     case kBootModem_Step7:								//
        stat = AdiCustomWrite(pHw, kADI_CMD_SET_MODE, kMODE_END_RESET, 0, NULL);	//
        if (stat!=USB_ST_NOERROR) wait=-1; 						//
        break;										//
     case kBootModem_Step8:								//
        myWord = HSTOLE(kMAILBOX_EMPTY);						//
        stat  = AdiCustomWrite(pHw, kADI_CMD_SET_2183_DATA, (kDSP_MP_TX_MAILBOX | 0x4000),//
                           2, (UInt8 *)&myWord);					//
        stat  = AdiCustomWrite(pHw, kADI_CMD_SET_2183_DATA, (kDSP_MP_RX_MAILBOX | 0x4000),//
                           2, (UInt8 *)&myWord);					//
        stat  = AdiCustomWrite(pHw, kADI_CMD_SET_2183_DATA, (kDSP_SWAP_MAILBOX  | 0x4000),//
                           2, (UInt8 *)&myWord);					//
        break;										//	
     case kBootModem_Step9:								//
        //delay 1000ms									//
        wait = MSEC_TO_JIFFIES(1000);							//
        break;										//
     case kBootModem_MainPage:								//
        IdmaUploadMainPage(pHw);  							//
        if (pHw->ApiTestReq==FALSE)							//
        {										//
          pHw->AdiModemSm.CurrentAdiState = STATE_BOOT_WAIT;				//
          SetTimerInterval(pHw, TRANS_TIME_4_BOOTWAIT);					//
        }										//
        wait = -1; 									//
        break;										//
     case kBootModem_EnableTrafic:							//
        break;										//
  }											//
											//	
  return wait;										//
}											//
											//
// BootMachiuneCompletionHandler sets the next next state and fires the timer to wake up//
// the booter thread after the time requested by Boot Machine				//
void BootMachineCompletionHandler(Hardware *ptr,long wait)				//
{											//
    Hardware* pHw = (Hardware*)ptr;							//
    int state=pHw->BootState;								//
											//
    switch (pHw->BootState)								//
    {											//
      case kBootModem_Kickoff:								//	
         state = kBootModem_Step2;							//
         break;										//
      case kBootModem_Step2:								//		
         state = kBootModem_Step3;							//
         break;										//
      case kBootModem_Step3:								//
         state = kBootModem_Step4; 							//	
         break;										//
      case kBootModem_Step4:								//
         // Wait 200ms									//
         state = kBootModem_Step5;							//
         break;										//
      case kBootModem_Step5:								//
         state = kBootModem_Step6;							//
         break;										//
      case kBootModem_Step6:								//
         // Wait 200ms									//
         state = kBootModem_Step7;							//
         break;										//
      case kBootModem_Step7:								//
         state = kBootModem_Step8;							//
         break;										//
      case kBootModem_Step8:								//
         state = kBootModem_Step9;							//
         break;										//
      case kBootModem_Step9:								//	
         state = kBootModem_MainPage;							//
         break;										//
      case kBootModem_MainPage:								//	
         break;										//
      case kBootModem_EnableTrafic:							//
         break;										//
    }											//
											//
    pHw->BootState = state;								//
    if (wait!=-1)									//
    {											//
      pHw->BootTimer.expires = jiffies+wait;						//
      add_timer(&pHw->BootTimer);							//
    } 											//
} 											//
											//
// Just sets the first step according to rebooting flag, sets API flags and wakes up    //
// booter thread									//
void BootTheModem(Hardware *pHw, Bool Rebooting)					//
{											//
											//
  //  By Mani ///////////////////////////////////////////////////////////		//	
  // In every boot start, we                                           //		//
  // 1- Make BlockSwapReq off. This flag only goes on when             //		//
  //    a test request is in progress and modem receives               //		//
  //    page number 1                                                  //		//
  // 2- Don't accepy User CMV Call by making UserCmvApiAcceptable      //		//
  //    FALSE                                                          //		//
  pHw->BlockSwapReq = FALSE;                                           //		//
  pHw->UserCmvApiAcceptable = FALSE;                                   //		//
  ///////////////////////////////////////////////////////////////////////		//
											//
  if (!Rebooting)									//
    pHw->BootState = kBootModem_Kickoff;						//
  else											//	
    pHw->BootState = kBootModem_Step3;							//
  wake_up_interruptible(&(pHw->wq2));							//
}											//
// End By Mani	//////////////////////////////////////////////////////////////////////////
	

/*********************************************************************************/
/* IdmaUploadMainPage								 */
/*										 */
/* Sends the Main page blocks of the DSP code through the IDMA interface.        */
/* Note that the bulk of this logic is straight from Buffer.c in the NDIS code,  */ 
/* there was virtually no documentation on how this is supposed to work.	 */
/*********************************************************************************/
static void IdmaUploadMainPage(Hardware *pHw)
{
    int stat;
    UInt32 i, myBlockSize;
    UInt8 *pLastBits = NULL;
    UInt8 *pBlockData;
    IDMALoadBlockInfo myBLI;

    ZAP(DEBUG_BOOT, DEBUG_ENTRY, ("+++IdmaUploadMainPage\n"));

    ZAP(DEBUG_BOOT, DEBUG_INFO, (kTAB"Loading %#x blocks from main page.\n",
        pHw->MainPage.BlockCount));

    for (i=0; i<pHw->MainPage.BlockCount; i++)
    {
        /*Init data that is not block-location-specific*/
        myBLI.ADIHeader = HSTOLE(0xABCD);
        myBLI.Ovl       = 0;
        myBLI.OvlOffset = HSTOLE(0x8000);
        myBLI.LastBlock = 0;

        /*Is this NOT the first block?*/
        if (pHw->MainPage.Blocks[i].DSPAddr != 0)
        {
           myBLI.Size  = HSTOLE((UInt16)(pHw->MainPage.Blocks[i].DSPSize));
           myBLI.Addr  = HSTOLE(pHw->MainPage.Blocks[i].DSPAddr);
           myBlockSize = (UInt16)(pHw->MainPage.Blocks[i].DSPSize);
           pBlockData  = pHw->MainPage.Blocks[i].MemOffset;
        }
        /*This is block 0 - we want to send all but offset 0 - so we start at offset 1*/
        /*We should only get here once!*/
        else
        {
           myBLI.Size  = HSTOLE((UInt16)(pHw->MainPage.Blocks[i].DSPSize - 3));
           myBLI.Addr  = HSTOLE(1);
           myBlockSize = (UInt16)(pHw->MainPage.Blocks[i].DSPSize - 3);
           pBlockData  = pHw->MainPage.Blocks[i].MemOffset+3;
           pLastBits   = pHw->MainPage.Blocks[i].MemOffset;
        }

        /*Write the structure telling the DSP what to expect in the next block*/
        stat = IDMAWrite(pHw, (void *)&myBLI, sizeof(IDMALoadBlockInfo));
        ZAPIF(stat != USB_ST_NOERROR, DEBUG_BOOT, DEBUG_ERROR,
              (kTAB"Error %#X on IDMA write of BlockLoadInfo\n", stat));

        /*********************************************************************************/
        /* Occassionally the modem will freak out during idma boot and get in 		 */
        /* a weird mode where it requests wrong swap pages, etc.			 */
        /* Comparing the usb traces of linux and Windows reveals that linux		 */
        /* does things MUCH faster, so my theory is that we're overwhelming		 */
        /* the hardware. So, I'll inject a tiny delay here to keep things		 */
        /* under control								 */
        /*********************************************************************************/
        wait_ms(1); 

        /*Now write the actual block data*/
        stat = IDMAWrite(pHw, (void *)pBlockData, myBlockSize);
        ZAPIF(stat != USB_ST_NOERROR, DEBUG_BOOT, DEBUG_ERROR,
              (kTAB"Error %#X on IDMA write of block data\n", stat));

        wait_ms(1);
    }

    /*All blocks are loaded, now we can write out offset 0 of the first block,    */
    /*which will tell the hardware to start - resulting in requests for swap pages*/
    myBLI.ADIHeader = HSTOLE(0xABCD);
    myBLI.Ovl       = 0;
    myBLI.OvlOffset = HSTOLE(0x8000);
    myBLI.LastBlock = HSTOLE(1);
    myBLI.Size      = HSTOLE(3);
    myBLI.Addr      = 0;
    myBlockSize     = 3;
    pBlockData      = pLastBits;
    ZAP(DEBUG_BOOT, DEBUG_INFO, (kTAB"sending last entry from main page, DSP will start soon!.\n"));

    /*Write the structure telling the DSP what to expect in the next block*/
    stat = IDMAWrite(pHw, (void *)&myBLI, sizeof(IDMALoadBlockInfo));
    ZAPIF(stat != USB_ST_NOERROR, DEBUG_BOOT, DEBUG_ERROR,
          (kTAB"Error %#X on IDMA write of BlockLoadInfo\n", stat));
    /*Now write the actual block data*/
    stat = IDMAWrite(pHw, (void *)pBlockData, myBlockSize);
    ZAPIF(stat != USB_ST_NOERROR, DEBUG_BOOT, DEBUG_ERROR,
          (kTAB"Error %#X on IDMA write of block data\n", stat));

    ZAP(DEBUG_BOOT, DEBUG_EXIT, ("---IdmaUploadMainPage\n"));
}

/*********************************************************************************/
/* IdmaUploadSwapPage								 */
/*										 */
/* Sends the blocks of the requested swap page through the IDMA interface.	 */
/* Note that the bulk of this logic is straight from Buffer.c in the NDIS code,  */
/* there was virtually no documentation on how this is supposed to work.	 */
/*********************************************************************************/
void IdmaUploadSwapPage(Hardware *pHw, UInt16 SwapInfo)
{
    int stat;
    UInt32 i, myBlockSize;
    UInt16 PageIndex, Ovl;
    IDMALoadBlockInfo myBLI;

    ZAP(DEBUG_BOOT, DEBUG_ENTRY, ("+++IdmaUploadSwapPage:\n"));

    PageIndex = (SwapInfo & 0x00FF) - 1;
    Ovl       = ((SwapInfo & 0xF000) >> 12) | ((SwapInfo & 0x0F00) >> 4);

    ZAP(DEBUG_BOOT, DEBUG_INFO, (kTAB"Loading %x blocks from swap page %x, ovl=%x.\n", 
                                 pHw->SwapPages[PageIndex].BlockCount, PageIndex, Ovl));

    if (PageIndex > pHw->SwapPageCount)
    {
       ZAP(DEBUG_BOOT, DEBUG_ERRORS, (kTAB"Request for swap page %x when only %x pages exist!\n",
                                      PageIndex, pHw->SwapPageCount));
       return;
    }
    
    /*For swap pages, we actually know we won't have more than 2 blocks*/
    for (i=0; i<pHw->SwapPages[PageIndex].BlockCount; i++)
    {
        /*Init data that is not block-location-specific*/
        myBLI.ADIHeader = HSTOLE(0xABCD);
        myBLI.Ovl       = HSTOLE(Ovl);
        myBLI.OvlOffset = HSTOLE(0x8000 | Ovl);
        myBlockSize     = pHw->SwapPages[PageIndex].Blocks[i].DSPExtendedSize;
        myBLI.Size      = HSTOLE((UInt16)myBlockSize);
        myBLI.Addr      = HSTOLE(pHw->SwapPages[PageIndex].Blocks[i].DSPAddr);

        if ((pHw->SwapPages[PageIndex].BlockCount-1) == i)
           myBLI.LastBlock = HSTOLE(1);
        else
           myBLI.LastBlock = HSTOLE(0);

        /*Write the structure telling the DSP what to expect in the next block*/
        stat = IDMAWrite(pHw, (void *)&myBLI, sizeof(IDMALoadBlockInfo));
        ZAPIF(stat != USB_ST_NOERROR, DEBUG_BOOT, DEBUG_ERROR,
              (kTAB"Error %#X on IDMA write of BlockLoadInfo\n", stat));

        /*********************************************************************************/
        /* Occassionally the modem will freak out during idma boot and get in 		 */
        /* a weird mode where it requests wrong swap pages, etc.			 */
        /* Comparing the usb traces of linux and Windows reveals that linux		 */
        /* does things MUCH faster, so my theory is that we're overwhelming		 */
        /* the hardware. So, I'll inject a tiny delay here to keep things		 */
        /* under control								 */
        /*********************************************************************************/
        wait_ms(1); 

        /*Now write the actual block data*/
        stat = IDMAWrite(pHw, (void *)(pHw->SwapPages[PageIndex].Blocks[i].MemOffset), myBlockSize);
        ZAPIF(stat != USB_ST_NOERROR, DEBUG_BOOT, DEBUG_ERROR,
              (kTAB"Error %#X on IDMA write of block data\n", stat));

        wait_ms(1);
    }
    //printk(KERN_EMERG "Swap Page Index requested: %d\n",PageIndex+1);
    // By Mani ////////////////////////////////////////////////////////////
    // when we get swap page number 1, if a test request is pending, we  //
    // set a flag (BlockSwapReq) to show that we want the swap page      //
    // request doesn't continue. In next time interval, state machine    //
    // handler checks the flag and sends CNTL.0.4 to put modem in test   //
    // if it has received "Modem Ready" message from modem.              //
    if ((PageIndex+1)==1)                                                //
    {                                                                    //
       if (pHw->ApiTestReq)                                              //
       {                                                                 // 
         pHw->BlockSwapReq = TRUE;                                       //
         pHw->AdiModemSm.ModemReplyExpected = 0xff;                      //
         pHw->UserCmvApiAcceptable = TRUE;                               //
         pHw->LastApiTime = jiffies;                                     //
         SetTimerInterval(pHw,100);                                      //
       }                                                                 // 
    }                                                                    //
    ///////////////////////////////////////////////////////////////////////

    ZAP(DEBUG_BOOT, DEBUG_EXIT, ("---IdmaUploadSwapPage\n"));
}


/*********************************************************************************/
/* IDMAWrite									 */
/*										 */
/* Responsible for sending data to the IDMA interface via the BULK IDMA OUT pipe.*/
/*********************************************************************************/
static int IDMAWrite(Hardware *pHw, void *pData, UInt32 Size)
{
    int ret = USB_ST_NOERROR;
    urb_t *urb;
    UInt8 *xfer_buf = GET_KBUFFER(Size);

    ZAP(DEBUG_BOOT, DEBUG_ENTRY, ("+++IDMAWrite\n"));

    if (!xfer_buf)
    {
       ZAP(DEBUG_BOOT, DEBUG_ERRORS, (kTAB"kmalloc in IDMAWrite for %#X bytes failed\n", Size));
       ret = -ENOMEM;
       goto IDMA_exit;
    }

    memcpy(xfer_buf, pData, Size);

    /* Get an URB and prepare it for submission*/
    urb = usb_alloc_urb(0);

    FILL_BULK_URB(urb, pHw->usbdev, pHw->pipeBulkIdmaOut, xfer_buf, Size, IDMACompletion, pHw);
    urb->transfer_flags |= USB_QUEUE_BULK;

    ZAP(DEBUG_BOOT, DEBUG_INFO, (kTAB"Submitting BULK URB %p\n", urb));

    ret = usb_submit_urb(urb);

IDMA_exit:
    ZAP(DEBUG_BOOT, DEBUG_EXIT, ("---IDMAWrite\n"));
    return ret;
}


/*********************************************************************************/
/* IDMACompletion								 */	
/*										 */
/* Called when an IDMAWrite urb has completed. All we have to do is free	 */
/* the memory allocated in IDMAWrite.						 */
/*********************************************************************************/
static void IDMACompletion(struct urb *urb)
{
    ZAP(DEBUG_BOOT, DEBUG_ENTRY, ("+++IDMACompletion\n"));

    /* All we really need to do is free the memory we alloced and free the urb*/
    FREE_KBUFFER(urb->transfer_buffer);
    usb_free_urb(urb);

    ZAP(DEBUG_BOOT, DEBUG_INFO, (kTAB"Freed BULK URB %p\n", urb));

    ZAP(DEBUG_BOOT, DEBUG_EXIT, ("---IDMACompletion\n"));
}

/*****************************************************************
$Log: Boot.c,v $

Revision 1.30  2002/05/24 15:27:00  Mani Hamzian
Adding support for User API calls

Revision 1.20  2002/05/24 21:59:30  Anoosh Naderi
Clean up the code

Revision 1.11  2002/01/14 21:59:30  chris.edgington
Added GPL header.

Revision 1.10  2002/01/08 16:27:47  chris.edgington
Fixed simple compile error.

Revision 1.9  2002/01/08 16:07:37  chris.edgington
Changed BootTheModem to be a void function.
Added some necessary comments.

Revision 1.8  2002/01/03 17:17:21  chris.edgington
Changed line-ending format to UNIX style.

Revision 1.7  2002/01/02 22:00:47  chris.edgington
Changed instances of STOLE macro to HSTOLE (Host Short To Little Endian).

Revision 1.6  2001/12/28 19:29:09  chris.edgington
Changed some debug messages from DEBUG_ATTN to DEBUG_INFO level.

Revision 1.5  2001/12/26 22:45:03  chris.edgington
Added "Rebooting" parm to BootTheModem.
Moved AdiCustomWrite to Utils.c.

Revision 1.4  2001/12/22 21:42:22  chris.edgington
Removed ifdef around setting of modem state after main page upload.

Revision 1.3  2001/12/22 19:51:34  chris.edgington
Added 1ms delay between blockinfo and blockdata to stabilize boot - linux just
does usb stuff too fast (relative to Windows).
Finished linux version of IdmaUploadSwapPage.
Rewrote IDMAWrite to use asynchronous completion of urbs.

Revision 1.2  2001/12/21 23:52:33  chris.edgington
Rewrote BootTheModem for Linux.
Rewrote IdmaUploadMainPage for Linux.
Added AdiCustomWrite.
Wrote IDMAWrite (needs more work - needs to pull urbs from a list.)
Added dump_urb.

Revision 1.1  2001/12/17 22:19:52  chris.edgington
Initial version - not part of build yet.

Revision 1.1  2001/12/10 03:27:03  chris.edgington
First version - ported from Boot.c in MACOS9 code.
********************************************************************************/












