/*******************************************************************************/
/* $Id: Stuff.h,v 1.50 2002/05/27 19:19:49 Anoosh Naderi Exp $ 		       */
/*									       */	
/* Copyright (c) 2002, Analog Devices Inc., All Rights Reserved		       */	
/*									       */	
/* Stuff.h								       */	
/*									       */
/* Miscellaneous stuff needed in the .c files of the application	       */
/* Much of this is required because of code that was used as-is from	       */
/* the MacOS9 project, or from the kernel driver code.			       */	
/*******************************************************************************/

/* We don't define ZAPS on purpose so we can do this:*/
#ifdef DEBUG
#define ZAP( _Module, _Level, _Msg) \
    printf _Msg;
#else
#define ZAP( _Module, _Level, _Msg)
#endif

#define UInt8                       unsigned char
#define UInt16                      unsigned short
#define UInt32                      unsigned int
#define Boolean                     unsigned int
#define OSStatus                    Boolean
#define FALSE                       0
#define false                       0
#define TRUE                        1
#define true                        1
#define ioErr                       1
#define noErr                       0

#define DSP_FIRMWARE_FILES          5
#define CFILE_ADSL_FIRMWARE0    "/etc/analog/rtbld0.bnm"
#define CFILE_ADSL_FIRMWARE1    "/etc/analog/rtbld1.bnm"
#define CFILE_ADSL_FIRMWARE2    "/etc/analog/rtbld2.bnm"
#define CFILE_ADSL_FIRMWARE3    "/etc/analog/rtbld3.bnm"
#define CFILE_ADSL_FIRMWARE4    "/etc/analog/rtbld4.bnm"

#define USB_PATH                "/proc/bus/usb/"

#define IFNULL_RETURN_MEMERR(ptr,msg) \
    if (ptr == NULL) \
    { \
        ZAP(DEBUG_INIT, DEBUG_ERRORS, (msg)); \
        return ioErr; \
    }

#define GET_MEMORY(size) \
    malloc(size);

#define GET_BUFFER GET_MEMORY

#define FREE_BUFFER(ptr) \
    free(ptr);

/* Just a simple structure to hold firmware data read from a file*/
typedef struct
{
    UInt8   *pBits;
    UInt32  Size;
} Firmware;

typedef struct
{
    /* Firmware data read from external files*/
    Firmware    FPGAFirmware;
    Firmware    ADSLFirmware[DSP_FIRMWARE_FILES];

    /* DSP code info*/
    UInt8      *pDsp;
    UInt32     IDMAStart;
} Hardware;

#define MAX_OPTION_NAME_LENGTH      64
typedef struct
{
	char 	Name[MAX_OPTION_NAME_LENGTH];
	UInt32	Value;
} DriverOption;

/* Device descriptor */
typedef struct {
	UInt8     bLength;
	UInt8     bDescriptorType;
	UInt16    bcdUSB;
	UInt8     bDeviceClass;
	UInt8     bDeviceSubClass;
	UInt8     bDeviceProtocol;
	UInt8     bMaxPacketSize0;
	UInt16    idVendor;
	UInt16    idProduct;
	UInt16    bcdDevice;
	UInt8     iManufacturer;
	UInt8     iProduct;
	UInt8     iSerialNumber;
	UInt8     bNumConfigurations;
} usb_device_descriptor;


/******************************************************************
$Log: Stuff.h,v $
Revision 1.50  2002/05/27 19:19:49  Anoosh Naderi
Clean up the code

Revision 1.4  2002/01/31 19:19:49  steve.urquhart
Modified usermode application to remove dependency on libusb.

Revision 1.3  2002/01/18 18:35:11  chris.edgington
Added ifdef DEBUG around ZAP
Added DriverOption struct.

Revision 1.2  2002/01/17 20:43:31  chris.edgington
Changed pathname to rtbld*.bnm files to /etc/analog.

Revision 1.1  2002/01/17 20:37:54  chris.edgington
First version.
***************************************************************/
